from ngsolve import *
from netgen.geom2d import unit_square
from math import pi

mesh = Mesh(unit_square.GenerateMesh(maxh = 0.1))
fes = H1(mesh, order = 1, dirichlet = 'left|bottom|right')

u, v = fes.TnT()

G = sin(pi * x)
H = -G
uex = (1 - y) * G
gradu = (pi * (1 - y) * cos(pi * x), H)
F = pi * pi * uex

gfu = GridFunction(fes)
gfu.Set(G, definedon = mesh.Boundaries('bottom'))

a = BilinearForm(fes)
a += grad(u) * grad(v) * dx
a.Assemble()

f = LinearForm(fes)
f += F * v * dx + H * v * ds ('top')
f.Assemble()

f.vec.data -= a.mat * gfu.vec
gfu.vec.data += a.mat.Inverse(freedofs = fes.FreeDofs()) * f.vec

gfuex = GridFunction(fes)
gfuex.Set(uex)

print("error 1: ", sqrt(Integrate((gfu - gfuex) ** 2, mesh)))
print("error 2: ", sqrt(Integrate((gfu - uex) ** 2, mesh, order = 3)))